@@ -1,95 +1,66 @@
 #' Computerized Multistage Testing (MST)
 #' @description \code{mst} creates a multistage (MST) object
-#' @param pool a data frame of item parameters
-#' @param design a numeric vector of the MST design (e.g., 1-2-3, or 1-2-2)
+#' @param pool the item pool (data.frame)
+#' @param design the MST design (string): e.g., "1-2-3", "1-2-2", etc.
 #' @param npanel the number of panels
-#' @param method the design method (i.e., 'topdown' or 'bottomup')
+#' @param method the design method: 'topdown' or 'bottomup'
 #' @param len the module/route length
 #' @param maxselect the maximum selection of items
 #' @details
-#' The \code{mst} object contains an item pool (\code{pool}), a test assembler (\code{assembler}), 
-#' a route map (\code{route}), a stage-module map (\code{module}), a design method (\code{method}),
-#' and several constants such as \code{npanel}, \code{nstage}, \code{nmodule}, \code{nroute}.\cr
-#' Two indices are used to index modules/testlets. \code{form} number is a unique identifier for all modules,
-#' which is used for automated test assembly. \code{index} is a panel-wise unique identifier, which is used for
-#' routing. \cr
-#' 
-#' There are two design methods for mst: \code{'bottomup'} and \code{'topdown'}. In the bottomup approach, 
-#' constraitns are imposed on individual modules. Conversely, in the topdown approach, constraints are imposed on routes.\cr
+#' The \code{mst} object contains: item pool (\code{pool}), ATA (\code{assembler}), 
+#' route map (\code{route}), module map (\code{module}), design method (\code{method}), 
+#' and several constants such as \code{npanel}, \code{nstage}, \code{nmodule}, 
+#' \code{nroute}.\cr
+#' Two identifiers are used to index modules/testlets. \code{form} is a unique 
+#' id for all modules (for ATA), and \code{index} is a panel-wise unique id (for MST).\cr
+#' There are two methods for designing MST: \code{'bottomup'} and \code{'topdown'}. 
+#' The bottomup approach adds objectives and constraints on individual modules, whereas
+#' the topdown approach adds objectives and constraints directly on routes.\cr
 #' @examples
 #' \dontrun{
-#' # generate a 300-item pool
-#' pool <- irt_model("3pl")$gendata(1,300)$items
+#' library(dplyr)
+#' set.seed(10001)
+#' ## generate item pool
+#' pool <- model_3pl()$gendata(1, 300)$items
 #' pool$content <- sample(1:3, nrow(pool), replace=TRUE)
 #' pool$time <- round(exp(rnorm(nrow(pool), log(60), .2)))
 #' 
 #' ## ex. 1: 1-2-2 MST, 2 panels, topdown
-#' ## 20 items, content = c(10, 5, 5)
-#' ## maximize information at -1 and 1 for easy and hard routes
-#' x <- mst(pool, design=c(1, 2, 2), npanel=2, method='topdown', len=20, maxselect=1)
-#' x <- mst_objective(x, theta=-1, indices=1:2)
-#' x <- mst_objective(x, theta= 1, indices=3:4)
-#' x <- mst_constraint(x, coef="content", min=10, max=10, level=1)
-#' x <- mst_constraint(x, coef="content", min=5, max=5, level=2)
-#' x <- mst_constraint(x, coef="content", min=5, max=5, level=3)
-#' x <- mst_stage_length(x, stages=c(1, 2, 3), min=1)
+#' ## 20 items in total, 10 items in content area 1
+#' ## maximize info. at -1 and 1 for easy and hard routes
+#' x <- mst(pool, "1-2-2", 2, 'topdown', len=20, maxselect=1)
+#' x <- mst_obj(x, theta=-1, indices=1:2)
+#' x <- mst_obj(x, theta=1, indices=3:4)
+#' x <- mst_constraint(x, "content", 10, 10, level=1)
 #' x <- mst_assemble(x, timeout=10)
-#' x$items
 #' plot(x)
 #' plot(x, byroute=TRUE)
 #' freq(mst_get_items(x, panel=1, route=1)$content, 1:3)$freq
 #' freq(mst_get_items(x, panel=2, route=4)$content, 1:3)$freq
 #' 
 #' ## ex. 2: 1-2-3 MST, 2 panels, bottomup, 
-#' ## 10 items in each module, content = c(4, 3, 3)
-#' ## maximize information at -1, 0 and 1 for easy, medium, and hard modules
-#' x <- mst(pool, design=c(1, 2, 3), npanel=2, method='bottomup', len=10, maxselect=1)
-#' x <- mst_route(x, c(1, 2, 6), "-")
-#' x <- mst_route(x, c(1, 3, 4), "-")
-#' x <- mst_objective(x, theta= 0, indices=1)
-#' x <- mst_objective(x, theta=-1, indices=c(2,4))
-#' x <- mst_objective(x, theta= 1, indices=c(3,5))
-#' x <- mst_constraint(x, coef="content", min=4, max=4, level=1)
-#' x <- mst_constraint(x, coef="content", min=3, max=3, level=2)
-#' x <- mst_constraint(x, coef="content", min=3, max=3, level=3)
+#' ## 10 items in total and 4 items in content area 1 in each module
+#' ## maximize info. at -1, 0 and 1 for easy, medium, and hard modules
+#' x <- mst(pool, "1-2-3", 2, 'bottomup', len=10, maxselect=1) %>%
+#'   mst_route(c(1, 2, 6), "-") %>%
+#'   mst_route(c(1, 3, 4), "-") %>%
+#'   mst_obj(theta= 0, indices=c(1, 5)) %>%
+#'   mst_obj(theta=-1, indices=c(2, 4)) %>%
+#'   mst_obj(theta= 1, indices=c(3, 6)) %>%
+#'   mst_constraint("content", 4, 4, level=1)
 #' x <- mst_assemble(x, timeout=10)
-#' plot(x)
-#' plot(x, byroute=TRUE)
-#' freq(mst_get_items(x, panel=1, route=1)$content, 1:3)$freq
-#' freq(mst_get_items(x, panel=2, route=4)$content, 1:3)$freq
-#' 
-#' ## ex. 3: 1-2-3 MST, 2 panels, topdown, 30 items, 
-#' ## 10 items in each content area
-#' ## target information at 18 at -1, 0, and 1 for easy, medium and hard routes
-#' x <- mst(pool, design=c(1, 2, 3), npanel=2, method='topdown', len=30, maxselect=1)
-#' x <- mst_route(x, c(1, 2, 6), "-")
-#' x <- mst_route(x, c(1, 3, 4), "-")
-#' x <- mst_objective(x, theta=-1, indices=1, target=16)
-#' x <- mst_objective(x, theta= 0, indices=2:3, target=16)
-#' x <- mst_objective(x, theta= 1, indices=4, target=16)
-#' x <- mst_constraint(x, coef="content", min=10, max=10, level=1)
-#' x <- mst_constraint(x, coef="content", min=10, max=10, level=2)
-#' x <- mst_constraint(x, coef="content", min=10, max=10, level=3)
-#' x <- mst_stage_length(x, stages=c(1, 2, 3), min=3)
-#' x <- mst_assemble(x, timeout=20)
-#' plot(x)
-#' plot(x, byroute=TRUE)
-#' freq(mst_get_items(x, panel=1, route=1)$content, 1:3)$freq
-#' freq(mst_get_items(x, panel=2, route=4)$content, 1:3)$freq
+#' group_by(x$items, panel, index) %>% summarise(n=sum(content==1))
 #' }
+#' @importFrom magrittr %>%
 #' @import lpSolveAPI
 #' @export
-mst <- function(pool, design, npanel, method, len=NULL, maxselect=NULL){
-  # validatioin
-  if(any(design < 1)) stop("invalid design.")
-  method <- tolower(gsub("[- ]", "", method))
-  if(!method %in% c('topdown', 'bottomup')) stop("invalid method. use 'topdown' or 'bottomup'")
-  
-  # Constants
+mst <- function(pool, design, npanel, method=c('topdown', 'bottomup'), len=NULL, maxselect=NULL){
+  method <- match.arg(method)
+  design <- strsplit(design, split="[-]") %>% unlist() %>% as.integer()
   nstage <- length(design)
   nmodule <- sum(design)
   
-  # Module-index map
+  # module-index map
   module <- NULL
   for(s in 1:nstage)
     for(m in 1:design[s])
@@ -105,96 +76,86 @@ mst <- function(pool, design, npanel, method, len=NULL, maxselect=NULL){
   nroute <- nrow(route)
   
   # ata
-  assembler <- ata(pool, npanel * nmodule)
-  x <- list(design=design, pool=pool, nitem=nrow(pool), npanel=npanel, 
-            nstage=nstage, nmodule=nmodule, nroute=nroute, module=module, 
-            route=route, assembler=assembler, method=method)
+  x <- list(pool=pool, nitem=nrow(pool), design=design, method=method, 
+            npanel=npanel, nstage=nstage, nmodule=nmodule, nroute=nroute, 
+            module=module, route=route, ata=ata(pool, npanel * nmodule))
   class(x) <- "mst"
   
   # constraint: test length
   if(!is.null(len) && length(len) == 1) x <- mst_constraint(x, 1, len, len)
   if(!is.null(len) && length(len) == 2) x <- mst_constraint(x, 1, len[1], len[2])
-  if(!is.null(len) && length(len) > 2) stop("invalid length. use 1 or 2 numbers.")
-  
+  if(!is.null(len) && length(len) > 2) stop("the length argument is too long.")
   # constraint: maxselect
-  if(!is.null(maxselect)) x$assembler <- ata_item_maxselect(x$assembler, maxselect)
+  if(!is.null(maxselect)) x$ata <- ata_item_maxselect(x$ata, maxselect)
+  # constraint: minimum stage length
+  x <- mst_stage_length(x, 1:x$nstage, min=1)
   
-  return(x)
+  x
 }
 
-
 #' @rdname mst
-#' @description \code{mst_route} adds and removes routes in the mst
-#' @param x the mst object
-#' @param route a vector of form indices to represent the route
+#' @description \code{mst_route} adds and removes routes in the MST
+#' @param x a MST object
+#' @param route a MST route represented by a vector of module indices 
 #' @param op \code{"+"} for adding a route and \code{"-"} for removing a route
 #' @export
-mst_route <- function(x, route, op){
+mst_route <- function(x, route, op=c("+", "-")){
   if(class(x) != "mst") stop("not a 'mst' object: ", class(x))
+  op <- match.arg(op)
   index <- apply(x$route[, 1:x$nstage], 1, function(r) all(r == route))
   
   if(op == "+") {
-    if(any(index)) stop("the route has been already added")
+    if(any(index)) 
+      stop("the route has been already added")
     if(!all(route %in% 1:x$nmodule))
-      stop("invalid route. index is out of bound.")
-    if(!all(with(x$module, stage[match(route, index)]) == 1:x$nstage))
-      stop("invalid route. make sure it consists of one module in each stage.")
+      stop("invalid route: module index is out of bound.")
     x$route <- rbind(x$route, c(route, NA))
   } else if(op == "-") {
-    if(!any(index)) stop("the route hasn't been added yet")
-    x$route <- x$route[-which(index), ]
-  } else {
-    stop("invalid operation. use '+' to add a new route or '-' to remove an existing route.")
+    if(!any(index))
+      stop("the route hasn't been added yet")
+    x$route <- x$route[!index, ]
   }
   
   x$nroute <- nrow(x$route)
   x <- mst_reindex_routes(x)
-  return(x)
+  x
 }
 
 
 #' @rdname mst
-#' @description \code{mst_objective} adds objective functions to the mst
+#' @description \code{mst_obj} adds objective functions to the MST
 #' @param theta a theta point or interval for optimization
 #' @param indices the index of the route (topdown) or the module (bottomup) for adding objectives
-#' @param target the target valus of the TIF objectives. \code{NULL} for maximization
+#' @param target the target values of the TIF objectives. \code{NULL} for maximization
 #' @param flatten the parameter for getting a flat TIF
-#' @param theta.step the step parameters to process \code{theta} interval
 #' @import lpSolveAPI
 #' @export
-mst_objective <- function(x, theta, indices=NULL, target=NULL, flatten=NULL, theta.step=2){
+mst_obj <- function(x, theta, indices=NULL, target=NULL, flatten=NULL) {
   if(class(x) != "mst") stop("not a 'mst' object: ", class(x))
   indices <- mst_indices_input(x, indices)
-  
-  if(length(theta) == 2) {
-    theta <- seq(theta[1], theta[2], length.out=theta.step + 2)
-    theta <- theta[-c(1, length(theta))]
-  } else if(length(theta) > 2) {
-    stop("invalid theta input. use one or two numbers to set TIF objective at a point or over an interval.")
-  }
   theta <- round(theta, 2)
   
   for(i in 1:x$npanel) {
     for(j in 1:nrow(indices)) {
       f <- unlist(indices[j, ]) + (i - 1) * x$nmodule
       if(is.null(target) || is.na(target)) {
-        x$assembler <- ata_obj_relative(x$assembler, theta, mode="max", flatten=flatten, forms=f, collapse=TRUE)
+        x$ata <- ata_obj_relative(x$ata, theta, mode="max", flatten=flatten, forms=f, collapse=TRUE)
       } else {
-        x$assembler <- ata_obj_absolute(x$assembler, theta, target=target, forms=f, collapse=TRUE)
+        x$ata <- ata_obj_absolute(x$ata, theta, target=target, forms=f, collapse=TRUE)
       }      
     }
   }
   
-  return(x)
+  x
 }
 
 
 #' @rdname mst
-#' @description \code{mst_constraint} adds constraints to the mst
+#' @description \code{mst_constraint} adds constraints to the MST
 #' @param coef the coefficients of the constraint
-#' @param level the level value for a categorical constraint
-#' @param min the minimum value of the contraint
-#' @param max the maximum value of the contraint
+#' @param level the constrained level, \code{NA} for continuous variable
+#' @param min the lower bound of the constraints
+#' @param max the upper bound of the constraints
 #' @import lpSolveAPI
 #' @export
 mst_constraint <- function(x, coef, min=NA, max=NA, level=NULL, indices=NULL){
@@ -204,11 +165,11 @@ mst_constraint <- function(x, coef, min=NA, max=NA, level=NULL, indices=NULL){
   for(i in 1:x$npanel){
     for(j in 1:nrow(indices)){
       f <- unlist(indices[j,] + (i - 1) * x$nmodule)
-      x$assembler <- ata_constraint(x$assembler, coef, min, max, level, forms=f, collapse=TRUE)
+      x$ata <- ata_constraint(x$ata, coef, min, max, level, forms=f, collapse=TRUE)
     }
   }
   
-  return(x)  
+  x
 }
 
 
@@ -228,62 +189,62 @@ mst_stage_length <- function(x, stages, min=NA, max=NA){
     if(!stages[i] %in% 1:x$nstage) stop("invalid stage input")
     f <- subset(x$module, x$module$stage == stages[i])$index
     f <- as.vector(outer(f, (1:x$npanel - 1) * x$nmodule, "+"))
-    x$assembler <- ata_constraint(x$assembler, 1, min[i], max[i], forms=f, collapse=FALSE)
+    x$ata <- ata_constraint(x$ata, 1, min[i], max[i], forms=f, collapse=FALSE)
   }
   
-  return(x)
+  x
 }
 
 
 #' @rdname mst
-#' @description \code{mst_set_rdp} anchors the routing decision point (rdp) between adjacent modules
+#' @description \code{mst_rdp} anchors the routing decision point (rdp) between adjacent modules
 #' @param tol tolerance parameter
 #' @import lpSolveAPI
 #' @export
-mst_set_rdp <- function(x, theta, indices, tol) {
-  # validation
+mst_rdp <- function(x, theta, indices, tol) {
   if(class(x) != "mst") stop("not a 'mst' object: ", class(x))
   if(length(theta) != 1) stop("rdp is not a single theta point")
   if(length(indices) != 2 || abs(indices[1] - indices[2]) != 1) stop("modules are not adjacent") 
   
-  info <- irt_stats(irt_model("3pl", theta=theta, items=x$pool), "info")
+  info <- irt_stats(model_3pl(theta=theta, items=x$pool), "info")
   info <- round(info[1,], 3)
   coef <- c(info, -1 * info)
   for(i in 1:x$npanel)
-    x$assembler <- ata_constraint(x$assembler, coef, -tol, tol, forms=indices + (i - 1) * x$nmodule, collapse=TRUE)
+    x$ata <- ata_constraint(x$ata, coef, -tol, tol, forms=indices + (i - 1) * x$nmodule, collapse=TRUE)
+  
   x
 }
 
 
 #' @rdname mst
 #' @description \code{mst_module_mininfo} sets the minimum information for modules
-#' @param mininfo the minimum information treshold
+#' @param mininfo the minimum information threshold
 #' @export
 mst_module_mininfo <- function(x, theta, mininfo, indices) {
   # validation
   if(class(x) != "mst") stop("not a 'mst' object: ", class(x))
   if(any(indices < 1 | indices > x$nmodule)) stop("invalid module index")
 
-  coef <- irt_stats(irt_model("3pl", t=theta, items=x$pool), "info")
+  coef <- irt_stats(model_3pl(theta=theta, items=x$pool), "info")
   coef <- round(coef, 3)
   for(i in 1:x$npanel)
     for(j in 1:length(theta))
-      x$assembler <- ata_constraint(x$assembler, coef[j, ], min=mininfo, forms=indices + (i - 1) * x$nmodule)
+      x$ata <- ata_constraint(x$ata, coef[j, ], min=mininfo, forms=indices + (i - 1) * x$nmodule)
+  
   x
 }
 
 
 #' @rdname mst
 #' @description \code{mst_assemble} assembles the mst
-#' @param solver the ata solver
-#' @param verbose the verbose parameter
 #' @import lpSolveAPI
 #' @export
-mst_assemble <- function(x, solver="lpsolve", verbose="none", ...){
+mst_assemble <- function(x, ...){
   if(class(x) != "mst") stop("not a 'mst' object: ", class(x))
-  x$assembler <- ata_solve(x$assembler, solver=solver, as.list=FALSE, verbose=verbose, ...)
-  if(!is.null(x$assembler$items)) {
-    items <- x$assembler$items
+  x$ata <- ata_solve(x$ata, as.list=FALSE, ...)
+  
+  if(!is.null(x$ata$items)) {
+    items <- x$ata$items
     items$panel <- ceiling(items$form / x$nmodule)
     items$index <- (items$form - 1) %% x$nmodule + 1
     items$stage <- x$module$stage[match(items$index, x$module$index)]
@@ -305,8 +266,8 @@ print.mst <- function(x, ...){
   print(x$module)
   cat("\n")
   cat("ATA assembles", x$npanel, "panel(s) from a pool of", x$nitem, "items:\n\n")
-  print(x$assembler)
-  cat("\n\n")
+  print(x$ata)
+  cat("\n")
   invisible(x)
 }
 
@@ -331,7 +292,7 @@ plot.mst <- function(x, ...){
       r <- subset(x$route, x$route$index == i)[1:x$nstage]
       for(j in 1:x$npanel){
         items <- subset(x$items, x$items$index %in% r & x$items$panel == j)
-        info <- irt_stats(irt_model("3pl", theta=opts$theta, items=items), "info", summary="people", fun=sum)
+        info <- irt_stats(model_3pl(theta=opts$theta, items=items), "info", summary="people", fun=sum)
         data <- rbind(data, data.frame(t=opts$theta, info=info, panel=j, route=i))
       }
     }
@@ -345,7 +306,7 @@ plot.mst <- function(x, ...){
   } else {
     for(i in unique(x$items$form)){
       items <- subset(x$items, x$items$form == i)
-      info <- irt_stats(irt_model("3pl", theta=opts$theta, items=items), "info", summary="people", fun=sum)
+      info <- irt_stats(model_3pl(theta=opts$theta, items=items), "info", summary="people", fun=sum)
       data <- rbind(data, data.frame(t=opts$theta, info=info, form=i, panel=items$panel[1], stage=items$stage[1], module=items$module[1]))
     }
     data$panel <- factor(paste("Panel", data$panel))
@@ -358,7 +319,7 @@ plot.mst <- function(x, ...){
       facet_grid(panel ~ stage)
   }
   
-  return(g)
+  g
 }
 
 #' @rdname mst
@@ -384,5 +345,5 @@ mst_get_items <- function(x, panel, stage=NULL, module=NULL, route=NULL){
     items <- subset(items, items$index %in% r)
   }
   
-  return(items)
+  items
 }
